/*
 * trace.c - tracing various os function calls
 *
 * Copyright (c) 2001 Laurent Vogel.
 *
 * Authors:
 *  LVL   Laurent Vogel
 *
 * This file is distributed under the GPL, version 2 or at your
 * option any later version.  See doc/license.txt for details.
 */

/* param meaning :
  w : word
  l : long
  p : pointer
  s : input string
*/

static struct {
  char *name,
  char *params,
} gemdos_spec[] = {
  /* 0x00 */    { "Pterm0", "" },
  /* 0x01 */    { "Cconin", "" },
  /* 0x02 */    { "Cconout", "w" },
  /* 0x03 */    { "Cauxin", "" },
  /* 0x04 */    { "Cauxout", "w" },
  /* 0x05 */    { "Cprnout", "w" },
  /* 0x06 */    { "Crawio", "w" },
  /* 0x07 */    { "Crawcin", "" },
  /* 0x08 */    { "Cnecin", "" },
  /* 0x09 */    { "Cconws", "s" },
  /* 0x0a */    { "Cconrs", "p" },
  /* 0x0b */    { "Cconis", "" },
  /* 0x0c */    NULL,
  /* 0x0d */    NULL,
  /* 0x0e */    { "Dsetdrv", "w" },
  /* 0x0f */    NULL,
  /* 0x10 */    { "Cconos", "" },
  /* 0x11 */    { "Cprnos", "" },
  /* 0x12 */    { "Cauxis", "" },
  /* 0x13 */    { "Cauxos", "" },
  /* 0x14 */    NULL,
  /* 0x15 */    NULL,
  /* 0x16 */    NULL,
  /* 0x17 */    NULL,
  /* 0x18 */    NULL,
  /* 0x19 */    { "Dgetdrv", "" },
  /* 0x1A */    { "Fsetdta", "p" },
  /* 0x1B */    NULL,
  /* 0x1C */    NULL,
  /* 0x1D */    NULL,
  /* 0x1E */    NULL,
  /* 0x1F */    NULL,
  /* 0x20 */    { "Super", "p" },
  /* 0x21 */    NULL,
  /* 0x22 */    NULL,
  /* 0x23 */    NULL,
  /* 0x24 */    NULL,
  /* 0x25 */    NULL,
  /* 0x26 */    NULL,
  /* 0x27 */    NULL,
  /* 0x28 */    NULL,
  /* 0x29 */    NULL,
  /* 0x2a */    { "Tgetdate", "" },
  /* 0x2b */    { "Tsetdate", "w" },
  /* 0x2c */    { "Tgettime", "" },
  /* 0x2d */    { "Tsettime", "w" },
  /* 0x2e */    NULL,
  /* 0x2f */    { "Fgetdta", "" },
  /* 0x30 */    { "Sversion", "" },
  /* 0x31 */    { "Ptermres", "lw" },
  /* 0x32 */    NULL,
  /* 0x33 */    NULL,
  /* 0x34 */    NULL,
  /* 0x35 */    NULL,
  /* 0x36 */    { "Dfree", "sw" },
  /* 0x37 */    NULL,
  /* 0x38 */    NULL,
  /* 0x39 */    { "Dcreate", "s" },
  /* 0x3a */    { "Ddelete", "s" },
  /* 0x3b */    { "Dsetpath", "s" },
  /* 0x3c */    { "Fcreate", "sw" },
  /* 0x3d */    { "Fopen", "sw" },
  /* 0x3e */    { "Fclose", "w" },
  /* 0x3f */    { "Fread", "wlp" },
  /* 0x40 */    { "Fwrite", "wlp" },
  /* 0x41 */    { "Fdelete", "s" },
  /* 0x42 */    { "Fseek", "lww" },
  /* 0x43 */    { "Fattrib", "sww" },
  /* 0x44 */    NULL,
  /* 0x45 */    { "Fdup", "w" },
  /* 0x46 */    { "Fforce", "ww" },
  /* 0x47 */    { "Dgetpath", "pw" },
  /* 0x48 */    { "Malloc", "l" },
  /* 0x49 */    { "Mfree", "l" },
  /* 0x4a */    { "Mshrink", "wll" },
  /* 0x4b */    { "Pexec", "ippp" },
  /* 0x4c */    { "Pterm", "w" },
  /* 0x4d */    NULL,
  /* 0x4e */    { "Fsfirst", "sw" },
  /* 0x4f */    { "Fsnext", "" },
  /* 0x50 */    NULL,
  /* 0x51 */    NULL,
  /* 0x52 */    NULL,
  /* 0x53 */    NULL,
  /* 0x54 */    NULL,
  /* 0x55 */    NULL,
  /* 0x56 */    { "Frename", "wss" },
  /* 0x57 */    { "Fdatime", "wpw" },
};

void trace_gemdos(void *stack)
{
/* ... */
}

